# coding=utf_8
# omezeni formatu fpak:
#	max file name length: 255
#	max file size 3.9GB

from cStringIO import StringIO
import os

try:
	from hashlib import md5
except:
	import md5

loaded_packs = {}
loaded_files = {}
HEAD_LEN = 8

def strToFsize(string):
	cnt = 0
	num = 0
	length = len(string)
	for character in string:
		v = ord(character)
		pwr = (length - cnt)-1
		num += v * (255**pwr)
		cnt+=1
	return num

def fsizeToStr(num):
	out = ""
	count = 0
	for n in xrange(4):
		pwr = 255**(4-n-1)
		v = (num-count) / pwr
		out += chr(int(v))
		count += int(v)*pwr
	return out

def openPak(fname):
	fname = os.path.normpath(fname)
	if os.path.exists(fname):
		if not fname in loaded_packs:
			return FPak(fname)
	else:
		print "fpak: file '%s' not found"%fname
		
def getFile(fname):
	if fname in loaded_files:
		return loaded_files[fname].read(fname)
	else:
		return False

class FPak:
	def __init__(self,filename,autoopen=True):
		self.filename = filename
		self.fp = None
		self.files = {}
		self.md = "NONE"
		self.offset = 0
		loaded_packs[self.filename] = self
		if autoopen:self._open()
		
	def __repr__(self):
		return "FPak '%s'"%(self.filename)
	
	def _open(self):
		self.fp = open(self.filename,"rb")
		
		if self.fp.read(HEAD_LEN) != "@KALFPAK":
			print "fpak error: invalid header in %s"%self.filename
			return False
		# delka hlavicky
		headlen = strToFsize(self.fp.read(4))
		self.offset = HEAD_LEN+4

		stored_files = []
		for i in xrange(headlen):
			# ctu delku nazvu
			file_name_length = ord(self.fp.read(1))
			# ctu jmeno souboru
			file_name = self.fp.read(file_name_length)
			# ctu delku souboru
			file_length = strToFsize(self.fp.read(4))
			# hodim do pole
			stored_files.append( (file_name,file_length) )
			self.offset += file_name_length+5
			
		self.md = self.fp.read(32)
		self.offset += 32
	
		file_start_offset = self.offset
		for f in stored_files:
			file_name,file_length = f
			loaded_files[file_name] = self
			self.files[file_name] = (file_start_offset,file_length)
			file_start_offset += file_length
		#print "fpak read: %s (%s files - %s kB)"%( self.filename, len(stored_files),int(file_start_offset/1024) )
		
	def close(self):
		self.files = []
		self.fp.close()
		
	def check(self):
		self.fp.seek(self.offset)
		data = self.fp.read()
		md = md5(data).hexdigest()
		if self.md == md:
			return True
		return False
		
	def read(self,fname,asfp=True):
		if fname in self.files:
			offset,length = self.files[fname]
			self.fp.seek(offset)
			data = self.fp.read(length)
			if asfp:
				return StringIO(data)
			else:
				return data
		else:
			return False
			
	def find(self,fname):
		return (fname in self.files)
		
	
class FPakEditable:
	def __init__(self,fname):
		self.files = {}
		self.fname = fname
		
	def put_data(self,fname,data):
		self.files[fname] = data
		
	def put_file(self,fname,fpath):
		fp = open(fpath,"rb")
		self.put_data(fname,fp.read())
		fp.close()
		
	def write(self):
		fp = open(self.fname,"wb")
		
		data = "@KALFPAK"
		
		head_data = []
		file_data = []
		headlen = 0
		for f in self.files:
			# prida hlavickova data
			fhead_len = chr(len(f))
			fdata = self.files[f]
			head_data.append( fhead_len + f + fsizeToStr(len(fdata)) )
			headlen += 1
			# prida data souboru
			file_data.append(fdata)
			print "fpak.write: add file '%s'"%f
			
		file_data = ''.join(file_data)
		print "fpak.write: md5 digest ..."
		md = md5(file_data).hexdigest()
		data += fsizeToStr(headlen)
		data += ''.join(head_data)
		data += md
		data += file_data
		print "fpak.write: writing ..."
		fp.write(data)
		fp.close()
		del data

