# coding=utf_8

from cStringIO import StringIO
import pygame

DATA_TYPE_STRING = 1
DATA_TYPE_LSTRING = 2
DATA_TYPE_LLSTRING = 3

DATA_TYPE_INT = 5
DATA_TYPE_LINT = 6
DATA_TYPE_LONG = 7
DATA_TYPE_FLOAT = 8

DATA_TYPE_MINT = 35
DATA_TYPE_MLINT = 36
DATA_TYPE_MLONG = 37
DATA_TYPE_MFLOAT = 38

DATA_TYPE_LIST = 10
DATA_TYPE_TUPLE = 11
DATA_TYPE_DICT = 12
DATA_TYPE_DICTKEY = 13

DATA_TYPE_SURFACE = 40

def encode(data):
	return enc_data(data)

def decode(data):
	if isinstance(data,str):
		data = StringIO(data)
	return dec_data(data)

def enc_str(string):
	data = ""
	length = len(string)
	if length > 255*255:
		data += chr(DATA_TYPE_LLSTRING)
		data += int2byte(length,4)
	elif length > 255:
		data += chr(DATA_TYPE_LSTRING)
		data += int2byte(length,2)
	else:
		data += chr(DATA_TYPE_STRING)
		data += int2byte(length,1)
	data += string
	return data
	
def enc_int(num):
	m = 0
	if num < 0:
		num = abs(num)
		m = 30
	data = ""
	if num > 255*255:
		data += chr(DATA_TYPE_LONG+m)
		data += int2byte(num,6)
	elif num > 255:
		data += chr(DATA_TYPE_LINT+m)
		data += int2byte(num,2)
	else:
		data += chr(DATA_TYPE_INT+m)
		data += int2byte(num,1)
	return data
	
def enc_float(num):
	m = 0
	if num < 0:
		num = abs(num)
		m = 30
	data = ""
	data += chr(DATA_TYPE_FLOAT+m)
	data += int2byte(len(str(num)),2)
	data += str(num)
	return data
	
def enc_list(arr):
	data = ""
	data += chr(DATA_TYPE_LIST)
	data += int2byte(len(arr),3)
	data += _enc_array(arr)
	return data

def enc_tuple(arr):
	data = ""
	data += chr(DATA_TYPE_TUPLE)
	data += int2byte(len(arr),3)
	data += _enc_array(arr)
	return data

def _enc_array(arr):
	data = ""
	for itm in arr:
		data += enc_data(itm)
	return data

def enc_dict(arr):
	data = ""
	data += chr(DATA_TYPE_DICT)
	data += int2byte(len(arr),3)
	for itm in arr:
		data += enc_data(itm)
		data += enc_data(arr[itm])
	return data

def enc_image(inp):
	data = ""
	data += chr(DATA_TYPE_SURFACE)
	w,h = inp.get_size()
	data += enc_data((pygame.image.tostring(inp,"RGB"),w,h))
	return data

def enc_data(inp):
	data = ""
	if isinstance(inp,tuple):
		data += enc_tuple(inp)
	elif isinstance(inp,list):
		data += enc_list(inp)
	elif isinstance(inp,dict):
		data += enc_dict(inp)
	elif isinstance(inp,str):
		data += enc_str(inp)
	elif isinstance(inp,unicode):
		data += enc_str(inp)
	elif isinstance(inp,float):
		data += enc_float(inp)
	elif isinstance(inp,int):
		data += enc_int(inp)
	elif isinstance(inp,long):
		data += enc_int(inp)
	elif isinstance(inp,pygame.Surface):
		data += enc_image(inp)
	else:
		print "fstruct: encode error - unknown data type %s"%type(inp)
	return data
	

def dec_data(data):
	typ = ord(data.read(1))

	if typ == DATA_TYPE_INT:
		return ord(data.read(1))
	elif typ == DATA_TYPE_LINT:
		return byte2int(data.read(2))
	elif typ == DATA_TYPE_LONG:
		return byte2int(data.read(6))

	elif typ == DATA_TYPE_MINT:
		return -ord(data.read(1))
	elif typ == DATA_TYPE_MLINT:
		return -byte2int(data.read(2))
	elif typ == DATA_TYPE_MLONG:
		return -byte2int(data.read(6))

	elif typ == DATA_TYPE_STRING:
		length = byte2int(data.read(1))
		return data.read(length)
	elif typ == DATA_TYPE_LSTRING:
		length = byte2int(data.read(2))
		return data.read(length)
	elif typ == DATA_TYPE_LLSTRING:
		length = byte2int(data.read(4))
		return data.read(length)
	
	elif typ == DATA_TYPE_FLOAT:
		length = byte2int(data.read(2))
		return float(data.read(length))
	elif typ == DATA_TYPE_MFLOAT:
		length = byte2int(data.read(2))
		return -float(data.read(length))
	
	elif typ == DATA_TYPE_TUPLE:
		return tuple(_dec_array(data))
	elif typ == DATA_TYPE_LIST:
		return _dec_array(data)
	elif typ == DATA_TYPE_DICT:
		return _dec_dict(data)
		
	elif typ == DATA_TYPE_SURFACE:
		return _dec_image(data)
	else:
		print "fstruct: decode error - unknown data type %s"%typ
		return None

def _dec_array(data):
	length = byte2int(data.read(3))
	out = []
	for n in xrange(length):
		d = dec_data(data)
		out.append(d)
	return out
	
def _dec_dict(data):
	length = byte2int(data.read(3))
	out = {}
	for n in xrange(length):
		key = dec_data(data)
		val = dec_data(data)
		out[key] = val
	return out

def _dec_image(data):
	st,w,h = dec_data(data)
	return pygame.image.frombuffer(st,(w,h),"RGB")

def int2byte(num,length=1):
	out = ""
	count = 0
	for n in xrange(length):
		pwr = 255**(length-n-1)
		v = (num-count) / pwr
		out += chr(int(v))
		count += int(v)*pwr
	return out
	
def byte2int(string):
	cnt = 0
	num = 0
	length = len(string)
	for character in string:
		v = ord(character)
		pwr = (length - cnt)-1
		num += v * (255**pwr)
		cnt+=1
	return num

