# -*- coding: utf-8 -*-
import socket
import thread
import time
import fstruct
PACKSIZE = 1000
class BaseNet:
	def get_ip(self):
		try:
			return [ip for ip in socket.gethostbyname_ex(socket.gethostname())[2] if not ip.startswith("127.")][0]
		except:
			return "?.?.?.?"
	
	def _recv(self,sock):
		data = None
		try:
			data = sock.recv(PACKSIZE+2)
			size = fstruct.byte2int(data[0:2])
			d = fstruct.decode(data[2:(2+size)])
			return d
		except Exception,e:
			err = "RECV: %s (%s)"%(e,data)
			print "RECV error: ",err
			self.callback("error",err)
			return False
		
	def _send(self,sock,data):
		try:
			data = fstruct.encode(data)
			if len(data) > PACKSIZE:
				raise IOError,"MPL: too large data pack (%s)"%len(data)
			_d = data.ljust(PACKSIZE)
			data = fstruct.int2byte(len(data),2)+""+_d
			return sock.send(data)
		except:
			return False
	

class Server(BaseNet):
	def __init__(self,ip,port,callback,bc=True):
		self.ip = ip
		self.name = "?"
		self.port = port
		self.callback = callback
		self.accepting = True
		self.broadcasting = bc
		self.socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
		try:
			self.socket.bind((self.ip, self.port))
			self.socket.listen(1)
		except Exception,e:
			self.callback("nobind",e)
		self._ip = self.get_ip()
		self.clients = []
		self.addrs = []
		thread.start_new_thread(self.broadcast, (0,))
		thread.start_new_thread(self.start, (0,))
	
	def broadcast(self,a=0):
		while self.broadcasting:
			sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
			sock.setsockopt(socket.SOL_SOCKET, socket.SO_BROADCAST,1)
			sock.sendto(fstruct.encode((self.name,self._ip)), ('<broadcast>' ,self.port))
			time.sleep(1)

	def send(self,data,exc=False):
		for sock,addr in self.clients:
			if addr == exc:continue
			self._send(sock,data)
			
	def start(self,a=0):
		while self.accepting:
			sock, addr = self.socket.accept()
			thread.start_new_thread(self.client_loop, (sock,addr))
			time.sleep(0.3)
			
	def close(self):
		self.socket.close()
		for s,a in self.clients:
			s.close()
			
	def client_loop(self,sock,addr=0):
		print "client connected from %s:%s"%addr
		self.clients.append((sock,addr))
		self.addrs.append(addr)
		self._send(sock,"@ADDR%s:%s"%addr)
		try:
			while True:
				data = self._recv(sock)
				if not data:
					self.client_disconnect(sock,addr)
					return False
				if data[0] == "@":
					if data[1:5] == "PING":self._send(sock,data)
				else:
					self.callback("data",(addr,data))
				time.sleep(0.005)
		except Exception,e:
			print "Client loop error: ",e
			self.client_disconnect(sock,addr)
	
	def client_disconnect(self,sock,addr):
		self.callback("disconnect",addr)
		self.clients.remove((sock,addr))
		self.addrs.remove(addr)
		sock.close()
		print "client disconnected"
	
		
			
class Client(BaseNet):
	def __init__(self,ip,port,callback):
		self.ip = ip
		self.port = port
		self.callback = callback
		self.connection = True
		self.connection_msg = ""
		self.running = True
		self.socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
		self.connect()
		
	def connect(self):
		print "connecting to %s:%s"%(self.ip,self.port)
		try:
			self.socket.connect((self.ip,self.port))
			print "connected"
			thread.start_new_thread(self.loop, (0,))
		except Exception,e:
			print "Client connecting error: ",e
			self.connection = False
			self.connection_msg = e
			self.running = False
			self.callback("error",e)
			
	def close(self):
		self.socket.close()
		
	def send(self,data):
		self._send(self.socket,data)
		
	def ping(self):
		self.send("@PING%s"%(int(time.time()*1000)))

	def tick(self):
		data = self._recv(self.socket)
		if not data:
			self.disconnect()
			return True
		if data[0] == "@":
			if data[1:5] == "PING":
				t = ( int(time.time()*1000) - int(data[5:]) )
				self.callback("ping",t)
			if data[1:5] == "ADDR":
				self.callback("addr",data[5:])
		else:
			self.callback("data",data)
		return False
		
	def loop(self,a=0):
		self.connection = True
		while self.running:
			self.tick()
			time.sleep(0.005)
			continue
			try:
				self.tick()
			except Exception,e:
				print "Client > Server error: ",e
				self.connection = False
				self.connection_msg = e
				self.callback("error","connection failed")
			time.sleep(0.005)
		self.disconnect()
		
	def disconnect(self):
		self.connection = False 
		self.socket.close()
		self.running = False

class BroadcastListener:
	def __init__(self,ip,port,callback):
		self.ip = ip
		self.port = port
		self.callback = callback
		self.listening = True
		self.socket = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
		self.socket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
		self.socket.setsockopt(socket.SOL_SOCKET, socket.SO_BROADCAST, 1)
		self.socket.bind((self.ip,self.port))
		thread.start_new_thread(self.loop, (0,))
		
	def close(self):
		self.listening = False
		self.socket.close()
	
	def loop(self,a=0):
		while self.listening:
			data,addr = self.socket.recvfrom(500)
			if data:
				data = fstruct.decode(data)
				self.callback((data,addr))
			time.sleep(0.2)
	
