# coding=utf_8
'''
	Kalaten RPG - http://kalaten.cz
	author: (c) 2011 Josef Vanžura 
	
	Licencováno pod GNU/LGPL.

	Tato knihovna je svobodný software; můžete ji šířit a upravovat
	v souladu s podmínkami GNU Lesser General Public License,
	tak jak ji vydala nadace Free Software Foundation; buď verze 2.1
	této licence, anebo (dle svého uvážení) kterékoli pozdější verze.

	Pro další podrobnosti viz GNU Lesser General Public License.

	Plné znění (anglicky) licence naleznete v souboru license.txt ve složce hry.
'''

import random,math
import pygame
import res

PARTICLE_LIMIT = 500
_particles_counter = 0
_particles_cache = {}
emitters = []

def update():
	global emitters
	for e in emitters:
		try:
			e.update()
			if e.killed:
				emitters.remove(e)
			e.draw()
		except:pass
		
def getParticlesCount():
	global emitters
	n = 0
	for e in emitters:
		n+= len(e.particles)
	return n

def _loadParticle(img,angle):
	global _particles_cache
	uid = "%s/%s"%(img,angle)
	if not uid in _particles_cache:
		image = res.cache.get(img).copy()
		image = pygame.transform.rotate(image,angle)
		_particles_cache[uid] = image
	return _particles_cache[uid]

def _particle_move_linear(pos,vector,life,cond):
	vx = math.sin(vector[0]*math.pi)*vector[1]
	vy = math.cos(vector[0]*math.pi)*vector[1]
	x = pos[0]+vx
	y = pos[1]+vy
	return (x,y)

def _particle_move_natural(pos,vector,life,cond):
	perc = (cond/life)
	vx = math.sin((perc)*vector[0]*(math.pi*2))*vector[1]
	vy = math.cos((perc)*vector[0]*(math.pi*2))*vector[1]
	x = pos[0]+vx
	y = pos[1]+vy
	return (x,y)

def _particle_move_quake(pos,vector,life,cond):
	vx = math.sin(vector[0]*math.pi)*(vector[1]*( 0.5-random.random() ))
	vy = math.cos(vector[0]*math.pi)*(vector[1]*random.random())
	x = pos[0]+vx
	y = pos[1]+vy
	return (x,y)

def _particle_move_fireworks(pos,vector,life,cond):
	perc = (cond/life)
	vx = math.sin(vector[0]*math.pi)*vector[1]*random.random()
	vy = math.cos(vector[0]*math.pi)*vector[1]*random.random()+((0.7-perc)*5)
	x = pos[0]+vx
	y = pos[1]+vy
	return (x,y)

def _particle_move_spir(pos,vector,life,cond):
	perc = math.tan(cond/life)
	vx = math.sin(perc*math.pi)*vector[1]
	vy = math.cos(perc*math.pi)*vector[1]
	x = pos[0]+vx
	y = pos[1]+vy
	return (x,y)
	
def _particle_move_spiral(pos,vector,life,cond):
	perc = (cond/life)
	p = (1-perc)*2
	vx = math.sin(vector[0]*p*math.pi*2)
	vy = math.cos(vector[0]*p*math.pi*2)
	x = pos[0]+(vx*vector[1])
	y = pos[1]+(vy*vector[1])
	return (x,y)

def _particle_move_implode(pos,vector,life,cond):
	perc = math.tan(cond/life)
	spd = vector[1]-(vector[1]*perc)

	vx = math.sin(vector[0]*math.pi)*spd
	vy = math.cos(vector[0]*math.pi)*spd
	x = pos[0]+vx
	y = pos[1]+vy
	return (x,y)

PARTICLE_MOVE_LINEAR = _particle_move_linear
PARTICLE_MOVE_QUAKE = _particle_move_quake
PARTICLE_MOVE_IMPLODE = _particle_move_implode
PARTICLE_MOVE_FIREWORKS = _particle_move_fireworks
PARTICLE_MOVE_SPIR = _particle_move_spir
PARTICLE_MOVE_NATURAL = _particle_move_natural
PARTICLE_MOVE_SPIRAL = _particle_move_spiral

class Particle:
	def __init__(self,emitter,start):
		global _particles_counter
		_particles_counter += 1
		self.e = emitter
		self.pos = start

		self.func = self.e.func
		self.set_vector()
		self.set_image()
		self.set_life()
		self.condition = self.life

		angle = int((self.vector[0])*180)
		image = _loadParticle(self.image,angle)
		self.sprite = pygame.sprite.DirtySprite(self.e.group)
		self.sprite.image = image
		self.sprite.rect = image.get_rect()
		
	def set_vector(self):
		vec = list(self.e.vector)
		if isinstance(vec[0],tuple):
			vec[0] = random.uniform(vec[0][0],vec[0][1])
		if isinstance(vec[1],tuple):
			vec[1] = random.uniform(vec[1][0],vec[1][1])
		self.vector = (vec[0],vec[1])
				
	def set_image(self):
		self.image = self.e.image
		if isinstance(self.image,tuple):
			self.image = random.choice(self.image)

	def set_life(self):				
		self.life = self.e.life
		if isinstance(self.life,tuple):
			self.life = int(random.uniform(self.life[0],self.life[1]))
						
	def update(self):
		if self.condition == 0:
			self.delete()
			return False
		self.pos = self.func(self.pos,self.vector,self.life,self.condition)
		self.sprite.rect.center = (int(self.pos[0]),int(self.pos[1]))
		self.condition += -1.0
		return True
		
	def delete(self):
		global _particles_counter
		_particles_counter += -1
		self.sprite.kill()
		del self.sprite

class Emitter:
	def __init__(self,surface,emcfg):
		self.surface = surface
		global emitters
		emitters.append(self)
		
		em = {
			"ppt": 1,
			"plimit":300,
			"func": PARTICLE_MOVE_LINEAR
		}
		em.update(emcfg)
		
		self.image = em["image"]
		self.life = em["life"]
		self.pos = em["pos"]
		self.pos_range = em["pos_range"]
		self.vector = ( em["vec_direction"],em["vec_speed"] )
		self.ppt = em["ppt"]
		self.plimit = em["plimit"]
		self.func = em["func"]
		
		self.autoemitt = False
		
		self.group = pygame.sprite.Group()
		
		self.particles = []
		self.deleted = False
		self.killed = False
		
	def emitt(self):
		for n in xrange(self.ppt):
			if len(self.particles) > self.plimit or _particles_counter > PARTICLE_LIMIT:
				break
			rxs,rxe = self.pos_range[0]
			rys,rye = self.pos_range[1]
			rx = self.pos[0]+random.uniform(rxs,rxe)
			ry = self.pos[1]+random.uniform(rys,rye)
			p = Particle(self,(rx,ry))
			self.particles.append(p)
			
	def delete(self):
		self.ppt = 0
		self.deleted = True
		
	def kill(self):
		global emitters
		self.group.empty()
		self.killed = True
		
	def update(self):
		if self.autoemitt:
			self.emitt()
		nlist = []
		append = nlist.append
		for p in self.particles:
			if p.update():append(p)
		self.particles = nlist
		if self.deleted and not len(nlist):
			self.kill()
			
	def draw(self):
		self.group.draw(self.surface)
		
	def set_vector(self,vec,refresh=True):
		self.vector = vec
		if refresh:
			for p in self.particles:
				p.set_vector()
		return True



