# coding=utf_8
import pygame
import fpak
import funclib
import bz2
import zlib
import os

class CacheManager:
	def __init__(self):
		self.data = {}
		pakname = os.path.normpath("data/graphics.dta")
		if os.path.exists(pakname):
			self.pak = fpak.openPak(pakname)
		else:
			self.pak = None
		
	def get(self,uid):
		if not uid in self.data:
			if self.pak:
				fp = self.pak.read("%s.ibi"%uid)
				self.data[uid] = readIBI(fp)
			else:
				img = pygame.image.load(os.path.normpath("graphics/%s.png"%uid))
				if img.get_bytesize() == 4:
					self.data[uid] = img.convert_alpha()
				else:
					self.data[uid] = img.convert()
		return self.data[uid]


cache = CacheManager()


def readIBI(fp,autoclose=True):
	''' Prevedeni IBI souboru do Surface '''
	# prectu hlavicku a overim ji
	header = fp.read(4)
	if header != "KIMG":return False
	# informace o datech
	width = funclib.byte2int(fp.read(2))
	height = funclib.byte2int(fp.read(2))
	format = int(fp.read(1))
	# prectu data a dekoduji
	data = fp.read()
	if format <= 1:
		data = bz2.decompress(data)
	else:
		data = zlib.decompress(data)
	if format in (0,2): data_format="RGB"
	if format in (1,3): data_format="RGBA"
	# prevedu data na Surface
	img = pygame.image.frombuffer(data,(width,height),data_format)
	# upravim podle alpha kanalu
	if format in (0,2):img = img.convert()
	if format in (1,3):img = img.convert_alpha()
	# a zvru pokud chci
	if autoclose:
		fp.close()
		del fp
	del data
	# ... hotovo
	return img
