# coding=utf_8
'''
	Kalaten RPG - http://kalaten.cz
	author: (c) 2011 Josef Vanžura 
	
	Licencováno pod GNU/LGPL.

	Tato knihovna je svobodný software; můžete ji šířit a upravovat
	v souladu s podmínkami GNU Lesser General Public License,
	tak jak ji vydala nadace Free Software Foundation; buď verze 2.1
	této licence, anebo (dle svého uvážení) kterékoli pozdější verze.

	Pro další podrobnosti viz GNU Lesser General Public License.

	Plné znění (anglicky) licence naleznete v souboru license.txt ve složce hry.
'''

import pygame,time,random,math
import resources,cfg,events,grid,glo
from const import *

music = None
MUSIC_CHECK_INTERVAL = 1000
MUSIC_MAX_SILENCE_LEN = 30 # maximalni delka ticha
SOUNDS_CLEAN_INTERVAL = 180*60*1000 # cistim zvuky po 3 minutach
RESERVE_CHANNELS = 3
CHANNELS_COUNT = 8
_syssnd = False

_sound_on = 1
_music_on = 1
_sound_volume = 1.0
_music_volume = 1.0

_sounds = {}
_usages = {}
_channels = {}

def init(_from_cfg=True):
	global _syssnd,_sound_on,_music_on,_sound_volume,_music_volume,music
	if _from_cfg:
		_sound_on = cfg.get("sound")
		_music_on = cfg.get("music")
		_sound_volume = cfg.get("sound_volume")
		_music_volume = cfg.get("music_volume")

	music = MusicManager(_music_volume)
	
	if _sound_on or _music_on:
		try:
			pygame.mixer.init(44100, -16, 2, 512)
			_syssnd = True
		except:
			print "sounds: audio system is not available"
			return False
	else:
		return False
		
	events.system.new(SYSEVENT_USEREVENT7,clear_sounds)
	pygame.time.set_timer(SYSEVENT_USEREVENT7,SOUNDS_CLEAN_INTERVAL)
	events.system.new(SYSEVENT_USEREVENT8,music.cycle)
	pygame.time.set_timer(SYSEVENT_USEREVENT8,MUSIC_CHECK_INTERVAL)
	
	pygame.mixer.set_num_channels(CHANNELS_COUNT)
	for ch in xrange(CHANNELS_COUNT):
		_channels[ch] = pygame.mixer.Channel(ch)
	return True

def quit():
	if not _syssnd:return False
	pygame.mixer.music.stop()
	pygame.mixer.quit()
	
def engine_mute():
	if not _syssnd:return False
	music.set_volume(0.5)
	pygame.mixer.pause()
	
def engine_unmute():
	if not _syssnd:return False
	music.set_volume(1.0)
	pygame.mixer.unpause()
	
	
def clear_sounds():
	'''
		Projde zvuky a zjisti kdy byly naposled prehrany/nacteny a pokud je to 
		dele nez SOUNDS_CLEAN_DELAY tak je uvolni. Vetsinou jede v intervalu.
	'''
	global _usages
	t = time.time()
	_nusages = {}
	print "sounds: clean"
	for s in _usages:
		delta = t-_usages[s]
		if delta >= SOUNDS_CLEAN_DELAY:
			print "sounds: clear sound '%s'"%s
			del _sounds[s]
		else:
			_nusages[s] = _usages[s]
	_usages = _nusages

######################################################### Sound Play
def load_sound(uid):
	''' Nacte zvuk pokud neexistuje vrati False jinak True. Zapisuje cas pristupu. '''
	ok = False
	if uid in _sounds:
		ok = True
	else:
		snd = resources.getSound(uid)
		if snd:
			_sounds[uid] = pygame.mixer.Sound(snd)
			ok = True
	if ok:
		# zvuk mam a ulozim si cas jeho prehrani + jeho delku
		snd_len = _sounds[uid].get_length()
		_usages[uid] = time.time()+snd_len
	return ok
	
@events.new_event
def play_sound(uid,volume=-1,loops=0,direction=0,force=False):
	'''
		Prehraje zvuk (automaticky ho nacte). Direction je smer zvuku -1.0 az 1.0
	'''
	if not _syssnd:return False
	if not _sound_on:return False
	if not load_sound(uid):
		print "sounds: sound file '%s' not found"%uid
		return False
	channel = pygame.mixer.find_channel(force)
	if not channel:
		print "sounds: no free channel for '%s'"%uid
		return False
	channel.play(_sounds[uid],loops)
	set_channel_volume(channel,volume,direction)
	return channel
	
def set_channel_volume(channel,volume=-1,direction=0):
	''' Nastavi volume kanalu i s pripadnym smerem zvuku. '''
	if not _syssnd:return False
	if not _sound_on:return False
	if volume == -1:volume = 1.0
	volume_left = 1.0
	volume_right = 1.0
	if direction:
		volume_left = 0.5 - float(direction)
		volume_right = 0.5 + float(direction)
		if volume_left > 1.0: volume_left = 1.0
		if volume_right > 1.0: volume_right = 1.0
		if volume_left < 0: volume_left = 0.0
		if volume_right < 0.0: volume_right = 0.0
	channel.set_volume(volume_left*volume,volume_right*volume)

######################################################### Music
class MusicManager:
	def __init__(self,volume):
		# [ ("track",mode,percent) ]
		self.tracks = {}
		self.master_volume = volume
		self.volume = 1.0
		self.playing = None
		self.lastplay = 0
		self.mode = 0
		self.last_mode = 0
		
	def put_track(self,uid,mode=0,perc=100):
		if not _syssnd:return False
		if not _music_on:return False
		self.tracks[uid] = MusicTrack(uid,mode,perc)
		
	def clear(self):
		self.tracks = {}

	def cycle(self):
		if not _syssnd:return False
		if not _music_on:return False
		busy = pygame.mixer.music.get_busy()
		if not busy:self.play()
		
	def play(self):
		if not _syssnd:return False
		if not _music_on:return False
		_toplay = []
		for uid in self.tracks:
			track = self.tracks[uid]
			if track.mode == self.mode:
				_toplay.append(track.uid)
		if _toplay:
			uid = random.choice(_toplay)
			if (time.time() - self.lastplay) > MUSIC_MAX_SILENCE_LEN:
				perc = 70
			else:
				perc = self.tracks[uid].perc*0.01
			if random.random() <= perc:
				self.play_track(uid)
				
		
	def play_track(self,uid):
		if not _syssnd:return False
		if not _music_on:return False
		print "sounds.music: play %s"%uid
		self.playing = uid
		self.lastplay = time.time()
		pygame.mixer.music.load( self.tracks[uid].path )
		pygame.mixer.music.play()
		self.set_volume(1.0)
		
	def set_volume(self,volume=1.0):
		if not _syssnd:return False
		if not _music_on:return False
		self.volume = volume
		pygame.mixer.music.set_volume(self.master_volume*self.volume)
		
	def fadeout(self,d=1500):
		if not _syssnd:return False
		if not _music_on:return False
		pygame.mixer.music.fadeout(d)
		self.playing = None
		
	def set_mode(self,mode,fade=800):
		''' Availiable modes: 0 - normal, 1 - battle, 2+ - USER '''
		if not _syssnd:return False
		if not _music_on:return False
		if self.mode != mode:
			self.lastplay = 0
			self.fadeout(fade)
			if mode == 1:
				self.last_mode = self.mode
		self.mode = mode
		
	def resume_mode(self):
		self.set_mode(self.last_mode)
		
		
class MusicTrack:
	def __init__(self,uid,mode,perc):
		self.uid = uid
		self.path = resources.getMusic(uid)
		self.mode = mode
		self.perc = perc
		
		
######################################################### Pseudo 2D 
sound_positions = {}
sound_positions_counter = 0
ambient_sounds = {}
ambient_sounds_counter = 0

class AmbientSound:
	def __init__(self,rid,vol=1.0,rand=False):
		global ambient_sounds_counter
		self.index = ambient_sounds_counter
		ambient_sounds_counter += 1
		self.rid = rid
		self.volume = vol
		self.rand = rand
		self.channel = False
		self.ts = 0
		ambient_sounds[self.index] = self
	
	def play(self):
		if not self.rand:
			self.channel = play_sound(self.rid,self.volume,-1)
		else:
			self.channel = play_sound(self.rid,self.volume,0)
			if self.channel:
				snd = self.channel.get_sound()
				self.ts = snd.get_length()+time.time()
			
	def check(self):
		if not self.channel:
			if self.rand:
				if random.random() < self.rand:
					self.play()
			else:
				self.play()
		elif self.rand and self.ts <= time.time():
			self.channel = False
	
	def stop(self):
		if self.channel:
			self.channel.stop()
	
	def remove(self):
		self.stop()
		del ambient_sounds[self.index]
		

class SoundSource:
	def __init__(self,res,sector,volume=1.0,typ=SOUNDSOURCE_MAP):
		global sound_positions_counter
		self.index = sound_positions_counter
		sound_positions_counter += 1
		self.res = res
		self.sector = sector
		self.volume = volume
		self.typ = typ
		sound_positions[self.index] = self
		
		self.x = -100
		self.y = -100
		self.channel = False
		self.active = False
		self.ts = 0

		x,y = events.call('maper.get_sector_posn',self.sector)
		self.x = x
		self.y = y
		
		self.check()

		
	def play(self):
		self.active = True
		if self.typ == SOUNDSOURCE_ONEPLAY:
			self.channel = play_sound(self.res,0,0)
			if not self.channel:
				self.remove()
				return False
			snd = self.channel.get_sound()
			self.ts = snd.get_length()+time.time()
		else:
			self.channel = play_sound(self.res,0,-1)
			if not self.channel:
				self.remove()
				return False
		self.update()
		
	def stop(self,t=0):
		if not self.channel:return False
		self.channel.stop()
		self.channel = -1
		self.active = False

	def update(self):
		if self.typ == SOUNDSOURCE_ONEPLAY:
			if self.ts <= time.time():
				self.remove()
				return False
		player_sec,pd = (glo.party_sector,glo.party_direction)
		px,py = events.call('maper.get_sector_posn',player_sec)
		vo,di = soundVolFromPosition((px,py),pd,(self.x,self.y),self.volume)
		set_channel_volume(self.channel,vo,di)

	def remove(self,t=0):
		self.stop(t)
		if self.index in sound_positions:
			del sound_positions[self.index]

	def check(self):
		player_sec,pd = (glo.party_sector,glo.party_direction)
		px,py = events.call('maper.get_sector_posn',player_sec)
		if grid.distance((self.x,self.y),(px,py)) < 15:
			if not self.active:
				self.play()
			else:
				self.update()
		elif self.active:
			self.stop()


def stopAllAmbientSounds():
	global ambient_sounds
	if not _syssnd:return False
	if not _sound_on:return False
	_tmp = {}
	for i in ambient_sounds:
		s = ambient_sounds[i]
		_tmp[i] = s
		s.stop()
	ambient_sounds = _tmp

def stopAllSoundSources(typ=SOUNDSOURCE_MAP):
	''' Zastavi vsechny zdroje zvuku '''
	global sound_positions
	if not _syssnd:return False
	if not _sound_on:return False
	_tmp = {}
	for i in sound_positions:
		s = sound_positions[i]
		if s.typ == typ:
			_tmp[i] = s
			s.stop()
	sound_positions = _tmp
	
def changeSoundsSources(sec,pd):
	''' Prepozicovani vsech zdroju zvuku '''
	if not _syssnd:return False
	if not _sound_on:return False
	for i in sound_positions:
		s = sound_positions[i]
		s.check()
	for i in ambient_sounds:
		s = ambient_sounds[i]
		s.check()

def soundVolFromPosition(listener_pos,listener_d,sound_pos,vol,t=0):
	''' Vraci hlasitosti kanalu pro zvuk na dane pozici vuci posluchaci '''
	px,py = listener_pos
	sx,sy = sound_pos
	x = (px-float(sx))
	y = (py-float(sy))
	if listener_pos == sound_pos:return (1.0,0.0)
	vol = vol * 0.98
	
	direction = (math.atan2(y, x)+math.pi) / math.pi
	
	if listener_d == "w":
		direction = direction
	elif listener_d == "e":
		direction += -1
	elif listener_d == "n":
		direction += -0.5
	elif listener_d == "s":
		direction += 0.5
	
	if direction < 0:
		direction=(2+direction)
	if direction > 2:
		direction=(direction-2)
	if direction < 0.5:
		direction=(0.5-direction)+0.5
		vol = vol * 0.6
	elif direction > 1.5:
		direction=1.5-(direction-1.5)
		vol = vol * 0.6
	direction = direction-1.0
	
	distance =  math.hypot(x,y)-1.5
	if distance < 0:distance=0
	silencer = 1.0 - ((distance**1.2)/10)
	if silencer < 0:silencer = 0.01
	vol = vol * silencer
	return vol,direction
	
