#! /usr/bin/env python
# coding=utf_8

import os,sys
sys.path.insert(0,"app")

import pygame
import math
import particles
import res
import grid
import unit
import glo
import mpl
import cfg
import fstruct
import gui
import random
import thread
import funclib
import transform

pygame.display.init()

pygame.font.init()
pygame.key.set_repeat(80,40)
cfg.init()
glo.ROCKETS_LIMIT = cfg.get("limit_rockets")
particles.PARTICLE_LIMIT = cfg.get("limit_particles")
pygame.display.set_caption("Batl v%s"%funclib.read_version())

_menu = True
try:
	for a in sys.argv:
		if a == "--debug":_menu=False
except:
	pass

if _menu:
	import menu
	if mpl.mpl.server:
		pygame.display.set_caption("Batl v%s Server"%funclib.read_version())
	else:
		pygame.display.set_caption("Batl v%s Client"%funclib.read_version())

else:
	pygame.display.set_caption("Batl v%s DEBUG"%funclib.read_version())
	glo.map_file = "data/8cross.map"
	glo.player_id = 0
	mpl.start_server()
	glo.lobby_clients.append(("Test",1))
glo.gui = {}
gui._gui_offset = 170


def load_map():
	glo.map_data = fstruct.decode(open(glo.map_file,"rb").read())

PPS = cfg.get("pps")
UPS = cfg.get("ups")
RESOLUTION = (900,700)
glo.resolution = RESOLUTION
load_map()
glo.screen = pygame.display.set_mode(RESOLUTION)
glo.screen_blend = pygame.Surface(RESOLUTION).convert()

class Player:
	def __init__(self,uid,name="Unnamed",group=1):
		self.uid = uid
		self.name = name
		self.group = group
		self.player = self
		self.money = glo.map_money
		self.harvesting_from = []

		self.base = True
		self.player_id = uid
		self.rank = 0
		self._fire = 0
		self.rotate_body = 0
		self.rotate_turret = 0
		self._goto = (0,0)
		self._targeting = ""
		self._class = ("tower","base")
		self.dead = False
		self.defense = 3
		self.size = (150/2)
		self.hp = 5000.0
		self.hp_max = 5000.0
		p = glo.map_data['players_pos'][self.uid]
		self.pos = ( (p[0]*50)+25,(p[1]*50)+25 )
		self.ipos = ((p[0]-1)*50,(p[1]-1)*50)
		self.baseimg = res.cache.get("base").copy()
		self.baseimg.blit(res.cache.get("player_colors"),(70,10),(0,uid*10,10,10))
		self.baseimg.blit(res.cache.get("player_colors"),(10,70),(0,uid*10,10,10))
		self.baseimg.blit(res.cache.get("player_colors"),(130,70),(0,uid*10,10,10))
		self.baseimg.blit(res.cache.get("player_colors"),(70,130),(0,uid*10,10,10))
		glo.units.append(self)
	
	def is_enemy(self,u):
		if self.group != u.player.group:
			return True
		return False

	def harm(self,unit,damage=0):
		if damage:
			dmg = damage
		else:
			dmg = unit.damage*(unit.get_rank()+1)
			dmg = dmg-self.defense
		if dmg<=0:return False
		self.hp += -dmg
		
	def kill(self):
		self.dead = True
		glo.order.insert(0,self.name)
		if len(glo.order) == len(glo.players)-1:
			winner = None
			for p in glo.players:
				if not p.dead:
					winner = p
					break
			glo.order.insert(0,winner.name)
			glo.theend = True
		unit.explosion(self.pos,3)
		for u in glo.units:
			if u.player_id == self.uid and not u.base:
				u.kill()

	
	def update(self):
		if self.hp <= 0 and not self.dead:
			self.hp = 0
			self.kill()
		else:
			self.harvesting_from = []
			self.money += 0.3
			self.hp += 1
			if self.hp >= self.hp_max:
				self.hp = self.hp_max

	def make_img(self):pass
	def goto(self,b=0,a=0):pass
	def blit(self,srf,ovrl):
		w,h = (150,150)
		pos = (int(round(self.pos[0]-(w/2))),int(round(self.pos[1]-(h/2))))
		cpos = (int(self.pos[0]),int(self.pos[1]))

		if not self.dead:
			q = (self.hp/float(self.hp_max))
			bpos = (
				pos[0]+((150-25)/2),
				pos[1]+((150-2)),
			)
			if q<0:q=0
			hp = int(q*25)
			pygame.draw.line(srf,(40,40,40),glo.relcoords(bpos),glo.relcoords((bpos[0]+25,bpos[1])),2)
			cl = (int(250*math.cos(q)), int( 250*math.sin(q) ), 50)
			pygame.draw.line(srf,cl,glo.relcoords(bpos),glo.relcoords((bpos[0]+hp,bpos[1])),2)

index = 0
for c,a in glo.lobby_clients:
	pl = Player(index,c,index)
	glo.players.append(pl)
	if index == glo.player_id:
		glo.iam = pl
	index += 1
	
if not _menu:
	glo.iam.money = 100000
	pl = glo.players[1]
	for u in ("fav1","fav1","fav1","fav1","fav2","fav2","fav2","fav2"):
		unit.add_unit(u,pl.pos,pl.uid)
	for pl in glo.players:
		pl.hp_max = 5000.0
		pl.hp = 5000.0

def select_home(noscroll=False):
	if not noscroll:
		p = glo.iam.pos
		glo.view_pos = [ p[0]-400, p[1]-300 ]
		check_viewpos()
	glo.selected = [glo.iam]

class SoftTimer:
	def __init__(self,func,intv):
		self.func = func
		self.intv = intv
		self.t = 0

	def tick(self,amm):
		self.t += amm
		if self.t >= self.intv:
			self.func()
		while self.t >= self.intv:
			self.t = self.t-self.intv


def update_effects():
	for eff in glo.effects:
		if glo.effects[eff] == 0:continue
		glo.effects[eff] += -1

def render_effects(surface):
	#glo.screen.blit(glo.screen,(0,0),None,pygame.BLEND_RGB_ADD)
	#transform.blur2(glo.screen)
	for eff in glo.effects:
		val = glo.effects[eff]
		if val <= 0:continue
		elif eff == "confuse":
			if val > 15:val = 15
			transform.deform(surface,1.0,val*3,5,5,0,True)
			transform.blur(surface,val,True)
		elif eff == "bloom":
			transform.bloom(surface,val,False)
		elif eff == "shake":
			if val > 10:val = 10
			transform.shake(surface,val,1.0)
		elif eff == "storm":
			transform.bloom(surface,val,False)

def render_messages(surface):
	if glo.theend:
		scr = res.cache.get("screen_end")
		x = (RESOLUTION[0]-scr.get_width())/2
		y = (RESOLUTION[1]-scr.get_height())/2
		glo.screen.blit(scr,(x,y),None,pygame.BLEND_RGB_SUB)
		x = (RESOLUTION[0]/2)
		y = (RESOLUTION[1]/2)
		txt = gui.renderString("The End",26,(200,200,200))
		glo.screen.blit(txt,(x-(txt.get_width()/2),y-35))
		index = 1
		for st in glo.order:
			siz = 22-(index*2)
			if siz < 12:siz = 12
			txt = gui.renderString("%s. %s"%(index,st),siz,(200,200,200))
			glo.screen.blit(txt,(x-(txt.get_width()/2),y))
			y += 25
			index += 1
	elif glo.messages:
		errscr = res.cache.get("screen_error")
		x = (RESOLUTION[0]-errscr.get_width())/2
		y = (RESOLUTION[1]-errscr.get_height())/2
		glo.screen.blit(errscr,(x,y),None,pygame.BLEND_RGB_SUB)
		x = (RESOLUTION[0]/2)
		y = (RESOLUTION[1]/2)
		txt = gui.renderString("Error",26,(255,0,0))
		glo.screen.blit(txt,(x-(txt.get_width()/2),y-35))
		for st in glo.messages:
			txt = gui.renderString(str(st),22,(255,0,0))
			glo.screen.blit(txt,(x-(txt.get_width()/2),y))
			y += 25


_tmpbg = None
_tmpbg_uid = ""

def paint_map():
	global cursor,_tmpbg,_tmpbg_uid
	glo.screen.fill((0,0,0))
	glo.screen_blend.fill((0,0,0,0))
	uid = "%s:%s"%(int(glo.view_pos[0]),int(glo.view_pos[1]))
	ox = (int(glo.view_pos[0]/50)*50)-int(glo.view_pos[0])
	oy = (int(glo.view_pos[1]/50)*50)-int(glo.view_pos[1])
	if _tmpbg_uid != uid:
		for _y in xrange(14+2):
			for _x in xrange(18+2):
				x = _x-1
				y = _y-1
				dx,dy = (glo.view_pos[0]/50),(glo.view_pos[1]/50)
				if (x+dx)<0 or (y+dy)<0:continue
				if (x+dx)>=len(glo.map_data['data'][0]) or (y+dy)>=len(glo.map_data['data']):continue
				v = glo.map_data['data'][int(y+dy)][int(x+dx)]
				if glo.tiles_anim[v] > 1:
					tile = res.cache.get("tile_%s_%s"%(glo.tiles[v],int( glo.tick%glo.tiles_anim[v] )) )
				else:
					tile = res.cache.get("tile_%s"%glo.tiles[v])
				glo.screen.blit(tile,(x*50+ox,y*50+oy))
		_tmpbg = glo.screen.copy()
		_tmpbg_uid = uid
	else:
		glo.screen.blit(_tmpbg,(0,0))
		for _y in xrange(14+2):
			for _x in xrange(18+2):
				x = _x-1
				y = _y-1
				dx,dy = (glo.view_pos[0]/50),(glo.view_pos[1]/50)
				if (x+dx)<0 or (y+dy)<0:continue
				if (x+dx)>=len(glo.map_data['data'][0]) or (y+dy)>=len(glo.map_data['data']):continue
				v = glo.map_data['data'][int(y+dy)][int(x+dx)]
				if glo.tiles_anim[v] > 1:
					tile = res.cache.get("tile_%s_%s"%(glo.tiles[v],int( glo.tick%glo.tiles_anim[v] )) )
					glo.screen.blit(tile,(x*50+ox,y*50+oy))	
	for p in glo.players:
		glo.screen.blit(p.baseimg,glo.relcoords(p.ipos))
		
	
	_nlist = []
	for u in glo.selected:
		if not u.dead:
			_nlist.append(u)
	glo.selected = _nlist

	vx,vy = glo.view_pos
	view_pos = (int(vx), int(vy))
	
	index = 0
	for c in glo.craters:
		frm,pos = glo.craters[c]
		if grid.pos_in_view(pos,view_pos,RESOLUTION):
			x,y = glo.relcoords(pos)
			glo.screen.blit(res.cache.get("crater_a%s"%(frm)),(x,y))
			if frm != 3:frm += 1
			glo.craters[c] = (frm,pos)
		index += 1
	

	for u in glo.units:
		if grid.pos_in_view(u.pos,view_pos,RESOLUTION):
			u.make_img()
	
	for u in glo.units:
		# kreslim mrtvolky
		if not u.dead:continue
		if not u._deadticks:continue
		if grid.pos_in_view(u.pos,view_pos,RESOLUTION):
			u.blit(glo.screen,glo.screen_blend)
			u._deadticks += -1
	
	for u in glo.units:
		# vykreslim zive jednotky
		if u.dead:continue
		u.blit(glo.screen,glo.screen_blend)

	num = 0
	for r in glo.rockets:
		if num == glo.ROCKETS_LIMIT:break
		r.blit(glo.screen,glo.screen_blend)
		num += 1
	
	if glo.cursor:
		x,y = glo.cursor_pos
		glo.screen.blit(res.cache.get("cur%s"%glo.cursor),glo.relcoords((x-25,y-25)),None,pygame.BLEND_RGB_ADD)
		glo.cursor += 1
		if glo.cursor >= 5:glo.cursor = 0
		
	if glo.selecting_start:
		sx,sy = glo.relcoords(glo.selecting_start)
		ex,ey = glo.relcoords(glo.selecting_end)
		pygame.draw.rect(glo.screen_blend,(0,10,20),(glo.relcoords(glo.selecting_start),( ex-sx, ey-sy )),0)
		pygame.draw.rect(glo.screen_blend,(20,40,60),(glo.relcoords(glo.selecting_start),( ex-sx, ey-sy )),1)
	
	if mpl.mpl.server:
		cli = mpl.mpl.get_clients()
		y = 5
		for c in cli:
			y += 14
			txt = gui.renderString("Client: %s"%c,12,(0,80,0))
			glo.screen_blend.blit(txt,(5,RESOLUTION[1]-y))
	else:
		if mpl.mpl.netw.connection:
			txt = "Connected to %s:%s"%(mpl.mpl.netw.ip,mpl.mpl.netw.port)
		else:
			txt = "Connection failed: %s"%mpl.mpl.netw.connection_msg
		txt = gui.renderString(txt,12,(0,180,0))
		glo.screen_blend.blit(txt,(5,RESOLUTION[1]-19))
			
	if glo.right_click:
		mx,my = pygame.mouse.get_pos()
		sx,sy = glo.right_click
		pygame.draw.line(glo.screen_blend,(50,60,10),(mx,my),(sx,sy),1)
		glo.view_pos[0] += (mx-sx)/5
		glo.view_pos[1] += (my-sy)/5
		check_viewpos()

def butclick(arg=0):
	print "clicked %s"%arg
	
def set_targeting(v=0,nonet=False):
	data = []
	for u in glo.selected:
		u.targeting(v)
		data.append(( u.uid, u._targeting ))
	if not nonet:
		mpl.netSend(("TARGET",data))


gui.add_button("build",select_home,(370,0),(45,30),True,True,False,"Build vehicles")
gui.add_button("menu",butclick,(475,0),(45,30),"menu",True,False,"Menu")

gui.add_button("target_rocket",set_targeting,(370,35),(40,35),"rocket",True,False,"Targeting: Rocket")
gui.add_button("target_tower",set_targeting,(410,35),(35,35),"tower",True,False,"Targeting: Towers/Base")
gui.add_button("target_heavy",set_targeting,(445,35),(35,35),"heavy",True,False,"Targeting: Heavy vehicles")
gui.add_button("target_ranked",set_targeting,(480,35),(40,35),"ranked",True,False,"Targeting: Ranked")

gui.add_button("target_slug",set_targeting,(370,70),(40,35),"slug",True,False,"Targeting: Cannons")
gui.add_button("target_tank",set_targeting,(410,70),(35,35),"tank",True,False,"Targeting: Vehicles")
gui.add_button("target_light",set_targeting,(445,70),(35,35),"light",True,False,"Targeting: Light vehicles")
gui.add_button("target_none",set_targeting,(480,70),(40,35),"none",True,False,"Targeting: None")

gui.add_button("mktower",unit.mktower,(320,35),(50,30),"none",True,False,"Build tower here")


def mousemove():
	p = pygame.mouse.get_pos()
	if gui.check_mouse_move(p):return
	x = p[0]+glo.view_pos[0]
	y = p[1]+glo.view_pos[1]
	if glo.selecting_start:
		sx,sy = glo.selecting_start
		glo.selecting_end = (x,y)
	#glo.units[0].goto((x,y))


def mousedown(b=0):
	p = pygame.mouse.get_pos()
	if gui.check_mouse_click(p):return
	x = p[0]+glo.view_pos[0]
	y = p[1]+glo.view_pos[1] 
	glo.right_click = False
	if 525+170<=p[0]<=625+170 and 5<=p[1]<=105:
		vx = ((p[0])-(525+170))/100.0
		vy = ((p[1])-5)/100.0
		px = int(glo.map_data["size"][0]*vx)
		py = int(glo.map_data["size"][1]*vy)
		wx = (px-8)*50
		wy = (py-6)*50
		glo.view_pos = [wx,wy]
		check_viewpos()
		if b == 3 and glo.selected:
			selected_goto(px*50,py*50)
	elif b == 1:
		glo.selecting_start = (x,y)
		glo.selecting_end = (x,y)
	if b == 3:
		glo.right_click = p
	#craters.append(( "a", 3, (x,y) ))

def selected_goto(x,y,nonet=False):
	if len(glo.selected) == 0:return 0
	data = []
	for u in glo.selected:
		r = u.size+len(glo.selected)
		dx = (r*random.random())-(r/2)
		dy = (r*random.random())-(r/2)
		tx = x+dx
		ty = y+dy
		u.goto((tx,ty))
		data.append(( u.uid,u._goto ))
	if not nonet:
		mpl.netSend(("GOTO",data))
		glo.cursor = 1
		glo.cursor_pos = (x,y)

def mouseup(b=0):
	p = pygame.mouse.get_pos()
	x = p[0]+glo.view_pos[0]
	y = p[1]+glo.view_pos[1]
	if b == 3:
		if glo.right_click:
			if grid.get_distance(pygame.mouse.get_pos(),glo.right_click) < 20:
				glo.selected = []
			glo.right_click = False
	if glo.selecting_start:
		_add = False
		if pygame.key.get_mods() in (1,2):_add = True
		sx,sy = glo.selecting_start
		ex,ey = (x,y)
		dx,dy = (abs(ex-sx),abs(ey-sy))
		if b == 1 and glo.selected and not _add:
			if dx > 40 or dy > 40:
				glo.selected = []
				glo.selecting_start = False
				glo.selecting_end = False

		
		if dx < 40 and dy < 40:
			if glo.selected:
				selected_goto(x,y)
				glo.selecting_start = False
				return False
			_d = 100
			_unit = None
			for u in glo.units:
				if u.dead:continue
				if u.player_id != glo.iam.uid:continue
				distance = grid.get_distance((x,y),u.pos)
				if distance < 30:
					if distance <= _d:
						_unit = u
						_d = distance
			if _unit:
				if _unit not in glo.selected:
					glo.selected.append(_unit)
			
		else:
			if not _add:glo.selected = []
			for u in glo.units:
				if u.dead:continue
				if u.base:continue
				if u.player_id != glo.iam.uid:continue
				ux,uy = u.pos
				if sx < ux < ex and sy < uy < ey:
					if not u in glo.selected:
						glo.selected.append(u)
			glo.selected = glo.selected[:15]
		if glo.iam in glo.selected:
			select_home()
		glo.selecting_start = False

def keydown(evt):
	k = evt.key
	if k == 276:#l
		glo.view_pos[0] += -25
	if k == 275:#r
		glo.view_pos[0] += 25
	if k == 273:#up
		glo.view_pos[1] += -25
	if k == 274:#dw
		glo.view_pos[1] += 25
	if k == 104:#H
		select_home()
	if k == 293:#F12
		glo.add_effect("confuse",420)
		#mpl.netSend(("EFFECT","confuse",120))
	check_viewpos()

def check_viewpos():
	w,h = glo.map_data['size']
	if glo.view_pos[0] <= 0:glo.view_pos[0] = 0
	if glo.view_pos[1] <= 0:glo.view_pos[1] = 0
	if glo.view_pos[0] >= (w-(18))*50:glo.view_pos[0] = (w-(18))*50
	if glo.view_pos[1] >= (h-(14))*50:glo.view_pos[1] = (h-(14))*50
	
def syscall(a=0,b=0):
	pass

def do_update(a=0):
	_clock = pygame.time.Clock()
	_clock_tick = _clock.tick
	try:
		while glo.running:
			mpl.mpl.tick()
			for u in glo.units:u.update()
			for r in glo.rockets:r.update()
			update_effects()
			_clock_tick(UPS)
			glo.tick += 1
	except Exception,e:
		print e
	glo.running = False

gui.make_building_buttons()
paint_map()
select_home()

thread.start_new_thread(do_update, (0,))


clock = pygame.time.Clock()
clock_tick = clock.tick
pygevget = pygame.event.get
running = True
while glo.running:
	tremains = clock_tick(PPS)
	evts = pygevget()
	for event in evts:
		if event.type == pygame.NOEVENT:break
		elif event.type == pygame.QUIT:
			glo.running = False
		elif event.type == pygame.ACTIVEEVENT:
			syscall(0,event.gain)
		elif event.type == pygame.MOUSEBUTTONUP:
			mouseup(event.button)
		elif event.type == pygame.MOUSEBUTTONDOWN:
			mousedown(event.button)
		elif event.type == pygame.MOUSEMOTION:
			mousemove()
		elif event.type == pygame.KEYDOWN:
			keydown(event)
	paint_map()
	particles.update()
	glo.screen.blit(glo.screen_blend,(0,0),None,pygame.BLEND_RGB_ADD)
	render_effects(glo.screen)
	gui.paint_minimap()
	gui.paint_ingame()
	render_messages(glo.screen)
	pygame.display.flip()
try:
	mpl.mpl.close()
except:
	pass
glo.running = False

