#!/bin/env python
# coding=utf_8


import os,sys
sys.path.insert(0,"app")
sys.path.insert(0,"data")

import pygame,math,res,grid,fstruct,glo

def renderString(text,size=12,color=(255,255,255)):
	font = pygame.font.Font("data/mono.ttf", size)
	return font.render(text,True,color)

gridon = True
mapon = False
view_pos = [0,0]
selected = 1
selected_pl = -1
painting = False
tooltip = ""
TILES = glo.tiles

def get_input(label,t="s",de=False):
	if t == "s":label += " (str)"
	if t == "i":label += " (int)"
	if t == "f":label += " (float)"
	if t == "b":label += " (a/n)"
	label += " [%s]"%de
	r = raw_input(label+": ")
	if not r: r = de
	if t == "i":r = int(r)
	if t == "f":r = float(r)
	if t == "b":
		if r in ("y","a"):r=1
		if r == "n":r=0
	return r

def save_map():
	global mapfile,mapdata
	mapdata['minimap'] = make_minimap()
	fp = open(mapfile,"wb")
	data = fstruct.encode(mapdata)
	fp.write(data)
	fp.close()
	print "map saved"

mapfile = None

try:
	mapfile = sys.argv[1]
except:
	print "No map file specified!"
	raise SystemExit
	
if os.path.exists(mapfile):
	print "Reading map file...",
	try:
		mapdata = fstruct.decode(open(mapfile,"rb").read())
		print "DONE"
	except:
		print "Broken map file."
		raise SystemExit
else:
	print "Map file doesn't exist. Will be created."
	mapdata = {
		"name": "",
		"players": 2,
		"size": (0,0),
		"minimap": None,
		"players_pos": {},
		"data":[]
	}
	
mapdata['name'] = get_input("Map name","s",mapdata['name'])
mapdata['players'] = get_input("Number of players","i",mapdata['players'])
if mapdata['size'][0] == 0:
	for p in xrange(mapdata['players']-1):
		mapdata['players_pos'][p+1] = (25,25)
	w = get_input("Map size","i",64)
	h = w
	if w <= 16 or h <= 16:
		print "Too small map."
		raise SystemExit
	mapdata['size'] = (w,h)
	for y in xrange(h):
		l = []
		for x in xrange(w):
			l.append(0)
		mapdata['data'].append(l)

###### INIT		
pygame.display.init()
screen = pygame.display.set_mode((640+35,640))
pygame.font.init()
pygame.key.set_repeat(80,40)
grid_img = res.cache.get("grid")
player_img = res.cache.get("player_colors")
tiles_img = []
tiles_img_s = []
for t in TILES:
	img = res.cache.get("tile_%s"%t)
	img = pygame.transform.smoothscale(img,(25,25))
	tiles_img.append(img)
	img = pygame.transform.smoothscale(img.copy(),(2,2))
	tiles_img_s.append(img)

###### EVENTS
def mouseup(b=0):
	global painting
	p = pygame.mouse.get_pos()
	if painting:
		painting = False

def mousedown(b=0):
	global selected,painting,selected_pl,view_pos
	p = pygame.mouse.get_pos()
	x = (p[0]/25)+view_pos[0]
	y = (p[1]/25)+view_pos[1]
	
	if mapon and p[0] < mapdata['size'][0]*2 and p[1] < mapdata['size'][1]*2:
		vx,vy = (p[0]/2,p[1]/2)
		view_pos = [vx-12,vy-12]
		check_viewpos()
	elif p[0] < 25*25 and p[1] < 25*25:
		if selected != -1:
			painting = True
			mapdata['data'][y][x] = selected
		else:
			mapdata['players_pos'][selected_pl] = (x,y)
	elif 625+15 <= p[0] <= 625+15+20 and 640-90 <= p[1] <= (640-90)+(mapdata['players']*10):
		selected = -1
		selected_pl = (p[1]-(640-90))/10
	else:
		th = len(tiles_img)*25
		if 625+10 <= p[0] <= 625+10+25 and 40<=p[1]<=40+th:
			v = (p[1]-40)/25
			selected = v
	pass

def mousemove():
	global tooltip,painting
	p = pygame.mouse.get_pos()
	x = (p[0]/25)+view_pos[0]
	y = (p[1]/25)+view_pos[1]
	if p[0] < 25*25 and p[1] < 25*25:
		if painting and selected != -1:
			mapdata['data'][y][x] = selected
		tooltip = "pos: %sx%s"%(x,y)
	else:
		tooltip = ""
	pass

def keydown(e):
	global gridon,mapon
	k = e.key
	print k
	if k == 103:
		gridon = not gridon
	if k == 115:
		save_map()
	if k == 109 and not mapon:
		mapon = True
	if k == 276:#l
		view_pos[0] += -1
		mapon = True
	if k == 275:#r
		view_pos[0] += 1
		mapon = True
	if k == 273:#up
		view_pos[1] += -1
		mapon = True
	if k == 274:#dw
		view_pos[1] += 1
		mapon = True
	check_viewpos()
	pass

def check_viewpos():
	if view_pos[0] > (mapdata['size'][0]-25):view_pos[0] = (mapdata['size'][0]-25)
	if view_pos[1] > (mapdata['size'][1]-25):view_pos[1] = (mapdata['size'][1]-25)
	if view_pos[0] <= 0:view_pos[0] = 0
	if view_pos[1] <= 0:view_pos[1] = 0


def keyup(e):
	global mapon
	k = e.key
	print k
	mapon = False
	if k == 109 and mapon:
		mapon = False


def draw_map():
	screen.fill((0,0,0))
	d = mapdata['data']
	for ry in xrange(25):
		for rx in xrange(25):
			x,y = (rx,ry)
			index = d[ry+view_pos[1]][rx+view_pos[0]]
			screen.blit(tiles_img[index],(x*25, y*25))
			if gridon:screen.blit(grid_img,(x*25, y*25),None,pygame.BLEND_RGB_ADD)
	
	ax,ay = (view_pos[0],view_pos[1])
	for p in mapdata['players_pos']:
		pos = mapdata['players_pos'][p]
		if ax <= pos[0] < ax+25 and ay <= pos[1] < ay+25:
			px,py = (pos[0]-ax, pos[1]-ay)
			screen.blit(player_img,(px*25, py*25),(0,10*p,10,10))
	
	draw_tools()

	if tooltip:
		txt = renderString(tooltip,12,(255,255,255))
		screen.blit(txt,(10,640-14))
	
	if mapon:
		draw_minimap()

def draw_tools():
	if selected != -1:
		img = tiles_img[selected]
		screen.blit(img,(625+10,10))
	y = 40
	for t in tiles_img:
		screen.blit(t,(625+10,y))
		screen.blit(grid_img,(625+10,y),None,pygame.BLEND_RGB_ADD)
		y += 25
	screen.blit(player_img,(625+15,640-90),(0,0,10,10*mapdata['players']))
	screen.blit(player_img,(625+15+10,640-90),(0,0,10,10*mapdata['players']))

def draw_minimap():
	d = mapdata['data']
	x = 0
	y = 0
	for line in d:
		for v in line:
			screen.blit(tiles_img_s[v],(x, y))
			x+=2
		x = 0
		y+=2
	for p in mapdata['players_pos']:
		pos = mapdata['players_pos'][p]
		screen.blit(player_img,(pos[0]*2,pos[1]*2), (0,p*10,2,2 ) )
	pygame.draw.rect(screen,(255,255,255),(view_pos[0]*2,view_pos[1]*2,50,50),1)

def make_minimap():
	d = mapdata['data']
	w = len(d[0])*2
	h = len(d)*2
	srf = pygame.Surface((w,h))
	x = 0
	y = 0
	for line in d:
		for v in line:
			srf.blit(tiles_img_s[v],(x, y))
			x+=2
		x = 0
		y+=2
	for p in mapdata['players_pos']:
		pos = mapdata['players_pos'][p]
		srf.blit(player_img,(pos[0]*2,pos[1]*2), (0,p*10,2,2 ) )
	srf = pygame.transform.smoothscale(srf,(100,100))
	return srf
			

def update():
	draw_map()
	pygame.display.flip()

clock = pygame.time.Clock()
clock_tick = clock.tick
pygevget = pygame.event.get
running = True
while running:
	tremains = clock_tick(20)
	evts = pygevget()
	for event in evts:
		if event.type == pygame.NOEVENT:break
		elif event.type == pygame.QUIT:
			running = False
		elif event.type == pygame.ACTIVEEVENT:
			pass
		elif event.type == pygame.MOUSEBUTTONUP:
			mouseup(event.button)
		elif event.type == pygame.MOUSEBUTTONDOWN:
			mousedown(event.button)
		elif event.type == pygame.MOUSEMOTION:
			mousemove()
		elif event.type == pygame.KEYDOWN:
			keydown(event)
		elif event.type == pygame.KEYUP:
			keyup(event)
	update()

pygame.quit()
